<?php declare(strict_types=1);

namespace EuKit\Base\Parts;

use Exception;
use Error;
use Infomaker\Everyware\NewsML\Objects\ContentPartItem;
use Infomaker\Imengine\Imengine;
use Infomaker\Everyware\Twig\View;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Support\Storage\Cache;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Base\Interfaces\NewsItemPresentation;
use Everyware\Plugin\SettingsParameters\SettingsParameter;

class ArticleBodyPresentation implements NewsItemPresentation
{
    const FB_API_APP_ID_KEY = 'FB_API_APP_ID_KEY';
    const FB_API_ACCESS_TOKEN_KEY = 'FB_API_ACCESS_TOKEN_KEY';
    const FB_API_200_CACHE_DURATION = 'FB_API_200_CACHE_DURATION';

    private $relatedArticles;

    public function __construct(PropertyObject $article)
    {
        $this->relatedArticles = [];

        foreach ($article->get('articles', []) as $relatedArticle) {
            $this->relatedArticles[$relatedArticle->uuid] = (object)[];
            if ($relatedArticle->permalink !== null) {
                $this->relatedArticles[$relatedArticle->uuid]->permalink = $relatedArticle->permalink;
            }
            if ($relatedArticle->teaser_image_uuids !== null && ! empty($relatedArticle->teaser_image_uuids)) {
                $this->relatedArticles[$relatedArticle->uuid]->thumbnail = $relatedArticle->teaser_image_uuids[0];
            }
            if ($relatedArticle->section !== null) {
                $this->relatedArticles[$relatedArticle->uuid]->section = $relatedArticle->section;
            }
        }
    }

    protected function element(Item $item): string
    {
        $content = $this->isListElement($item) ? $this->generateListContent($item) : $item->get('content');

        return $this->generateArticlePart('element', [
            'type' => str_replace('x-im/', '', $item->getType()),
            'content' => $content ? htmlspecialchars_decode($content) : '',
        ]);
    }

    protected function image(ImageItem $item): string
    {
        $image = new Image($item);

        return $this->generateArticlePart('image', $image->getViewData());
    }

    protected function table(Item $item): string
    {
        return $this->generateArticlePart('table', $item->toArray());
    }

    protected function htmlEmbed(Item $item): string
    {
        return $this->generateArticlePart('html-embed', $item->toArray());
    }

    protected function imageGallery(Item $item): string
    {
        $images = array_map([&$this, 'image'], (array)$item->get('images'));

        $item->set('images', $images);

        return $this->generateArticlePart('gallery', $item->toArray());
    }

    protected function contentPart(Item $item): string
    {
        $image = null;
        if ($item instanceof ContentPartItem && $item->hasImage()) {
            $image = new Image($item->getImage());
            $image->setAutomaticCropEnabled(false);
        }
        $contentPartText = array_map([&$this, 'element'], (array)$item->get('text'));
        $item->fill([
            'image' => $image instanceof Image ? $image->getViewData() : null,
            'text' => implode('', $contentPartText)
        ]);

        return $this->generateArticlePart('content-part', $item->toArray());
    }

    protected function relatedArticle(Item $item): string
    {
        if ( ! isset($item->uuid)) {
            return '';
        }

        if (isset($this->relatedArticles[$item->uuid])) {
            $item->set('permalink', $this->relatedArticles[$item->uuid]->permalink);
            $item->set('section', $this->relatedArticles[$item->uuid]->section);

            if (isset($this->relatedArticles[$item->uuid]->thumbnail)) {
                $size = 100;
                $thumbnail = new SimpleImage([
                   'src' => Imengine::create()->fit($size)->fromUuid($this->relatedArticles[$item->uuid]->thumbnail),
                   'width' => $size,
                   'height' => $size,
                ]);
                $item->set('thumbnail', $thumbnail);
            }

            return $this->generateArticlePart('related-article', $item->toArray());
        }

        return '';
    }

    protected function pdf(Item $item): string
    {
        return $this->generateArticlePart('pdf', [
            'url' => $item->get('url'),
            'title' => $item->get('title'),
        ]);
    }

    protected function youtube(Item $item): string
    {
        return $this->generateArticlePart('youtube', $item->toArray());
    }

    protected function review(Item $item): string
    {
        $text = array_map([&$this, 'element'], (array)$item->get('text'));

        $item->set('text', implode('', $text));

        return $this->generateArticlePart('review', $item->toArray());
    }

    protected function youplay(Item $item): string
    {
        return $this->generateArticlePart('youplay', $item->toArray());
    }

    protected function socialEmbed(Item $item): ?string
    {
        $links = $item->get('content');

        if (empty($links)) {
            return '';
        }

        if ( ! ($linkItem = array_shift($links)) instanceof Item) {
            return '';
        }
        switch ($linkItem->getType()) {
            case 'x-im/tweet':
                return $this->twitter($linkItem);
            case 'x-im/facebook-post':
            case 'x-im/facebook-video':
            case 'x-im/facebook-watch':
            case 'x-im/facebook-page':
                return $this->facebook($linkItem);
            case 'x-im/instagram':
                return $this->instagram($linkItem);
            case 'x-im/vimeo':
                return $this->vimeo($linkItem);
            default:
                return '';
        }
    }

    protected function instagram(Item $item): string
    {
        $embed_url = $item->embed_url ?? $item->url ?? '';
        $key = md5("instagram_{$embed_url}");
        $embedHtml = Cache::get($key) ?: '';


        $appId = SettingsParameter::getValue(self::FB_API_APP_ID_KEY) ?? '';
        $accessToken = SettingsParameter::getValue(self::FB_API_ACCESS_TOKEN_KEY) ?? '';

        $timeToCache200 = SettingsParameter::getValue(self::FB_API_200_CACHE_DURATION) ?? 60;

        if (empty($embedHtml)) {
            try {
                if (isset($item->url)) {
                    $apiResponse = wp_remote_get('https://graph.facebook.com/v8.0/instagram_oembed?url=' . $item->url . '&access_token=' . $appId . '|' . $accessToken);

                    if ($apiResponse['response']['code'] === 200) {
                        $embedHtml = json_decode($apiResponse['body'], false)->html;
                        Cache::set($key, $embedHtml, $timeToCache200 * MINUTE_IN_SECONDS);
                    } else {
                        Cache::set($key, $embedHtml, 1 * MINUTE_IN_SECONDS);
                    }
                }
            } catch (Error $e) {
                return '';
            } catch (Exception $e) {
                return '';
            }
        }

        return $this->generateArticlePart('instagram', [
            'content' => $embedHtml
        ]);
    }

    protected function facebook(Item $item): string
    {
        $templateName = str_replace('x-im/', '', $item->getType());

        return $this->generateArticlePart($templateName, [
            'url' => $item->url ?? ''
        ]);
    }

    protected function twitter(Item $item): string
    {
        return $this->generateArticlePart('twitter', [
            'url' => $item->url ?? ''
        ]);
    }

    protected function vimeo(Item $item): string
    {
        if ( ! isset($item->uri)) {
            return '';
        }

        return $this->generateArticlePart('vimeo', [
            'vimeo_id' => $vimeo_id = str_replace('im://vimeo/', '', $item->uri),
            'url' => 'https://player.vimeo.com/video/' . $vimeo_id,
            'embedId' => $item->embedId,
        ]);
    }

    protected function iframely(Item $item): string
    {
        return $this->generateArticlePart('iframely', [
            'content' => $item->content ?? '',
            'links' => $item->links ?? []
        ]);
    }

    protected function generateListContent(Item $item): string
    {
        if ( ! $this->isListElement($item)) {
            return '';
        }

        $listItems = str_replace('list-item', 'li', $item->getFilteredContent(['a', 'list-item']));

        switch ($item->getType()) {
            case 'x-im/unordered-list':
                return "<ul>{$listItems}</ul>";
            case 'x-im/ordered-list':
                return "<ol>{$listItems}</ol>";
        }

        return '';
    }

    protected function isListElement(Item $item)
    {
        $validListTypes = ['x-im/unordered-list', 'x-im/ordered-list'];

        return in_array($item->getType(), $validListTypes, true);
    }

    protected function generateArticlePart($part, $data): string
    {
        try {
            return View::generate("@base/article/{$part}", $data);
        } catch (Exception $e) {
            NewRelicLog::error('Failed to generate article body part', $e);
        }

        return '';
    }

    public function generateItem(string $type, Item $item): string
    {
        switch ($type) {
            case 'x-im/youtube':
                return $this->youtube($item);
            case 'x-im/youplay':
                return $this->youplay($item);
            case 'x-im/image':
                return $item instanceof ImageItem ? $this->image($item) : '';
            case 'x-im/link':
                return implode('', $item->toArray());
            case 'x-im/article':
                return $this->relatedArticle($item);
            case 'x-gm/review':
                return $this->review($item);
            case 'x-im/socialembed':
                return $this->socialEmbed($item);
            case 'x-im/content-part':
                return $this->contentPart($item);
            case 'x-im/htmlembed':
                return $this->htmlEmbed($item);
            case 'x-im/imagegallery':
                return $this->imageGallery($item);
            case 'x-im/table':
                return $this->table($item);
            case 'x-im/pdf':
                return $this->pdf($item);
            case 'x-im/iframely':
                return $this->iframely($item);
            default:
                return $this->element($item);

        }
    }
}
